@@ -30,13 +30,14 @@
 #define CPUFREQ_MONITOR_INTERVAL 1
 
 #ifdef HAVE_GET_FREQUENCIES
-typedef struct cpufreq_frequencies cpufreq_available_frequencies;
+typedef struct cpufreq_frequencies CPUFreqFrequencyList;
 #define cpufreq_get_available_frequencies(cpu) cpufreq_get_frequencies ("available", cpu)
 #define cpufreq_put_available_frequencies(first) cpufreq_put_frequencies (first)
+#else
+typedef struct cpufreq_available_frequencies CPUFreqFrequencyList;
 #endif
 
 typedef struct cpufreq_policy                CPUFreqPolicy;
-typedef struct cpufreq_available_frequencies CPUFreqFrequencyList;
 typedef struct cpufreq_available_governors   CPUFreqGovernorList;
 
 struct _CPUFreqMonitor
