@@ -23,10 +23,12 @@
 
 #include <libwnck/libwnck.h>
 #include <panel-applet.h>
+#include <X11/Xlib.h>
 
 struct _TaskListPrivate {
     WnckScreen *screen;
     WpApplet *windowPickerApplet;
+    guint size_update_event_source;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (TaskList, task_list, GTK_TYPE_BOX);
@@ -34,11 +36,11 @@ G_DEFINE_TYPE_WITH_PRIVATE (TaskList, task_list, GTK_TYPE_BOX);
 static GSList *task_lists;
 
 static TaskList *
-get_task_list_for_monitor (TaskList *task_list,
-                           gint      monitor)
+get_task_list_for_monitor (TaskList   *task_list,
+                           GdkMonitor *monitor)
 {
     GSList *list;
-    gint list_monitor;
+    GdkMonitor *list_monitor;
 
     list = task_lists;
 
@@ -61,26 +63,23 @@ static void on_task_item_closed (
     TaskItem *item,
     TaskList *list)
 {
-    gtk_container_remove (
-        GTK_CONTAINER (list),
-        GTK_WIDGET (item)
-    );
-    gtk_widget_destroy (GTK_WIDGET (item));
+    gtk_container_remove (GTK_CONTAINER (list),
+                          GTK_WIDGET (item));
 }
 
-static gint
+static GdkMonitor *
 window_get_monitor (WnckWindow *window)
 {
-    GdkScreen *gdk_screen;
+    GdkDisplay *gdk_display;
     gint x, y, w, h;
 
-    gdk_screen = gdk_screen_get_default ();
+    gdk_display = gdk_display_get_default ();
 
     wnck_window_get_geometry (window, &x, &y, &w, &h);
 
-    return gdk_screen_get_monitor_at_point (gdk_screen,
-                                            x + w / 2,
-                                            y + h / 2);
+    return gdk_display_get_monitor_at_point (gdk_display,
+                                             x + w / 2,
+                                             y + h / 2);
 }
 
 static void
@@ -88,8 +87,8 @@ on_task_item_monitor_changed_cb (TaskItem *item,
                                  gint      old_monitor,
                                  TaskList *current_list)
 {
-    gint monitor;
-    gint list_monitor;
+    GdkMonitor *monitor;
+    GdkMonitor *list_monitor;
     TaskList *list;
 
     monitor = task_item_get_monitor (item);
@@ -123,8 +122,8 @@ static void create_task_item (TaskList   *taskList,
                               WnckWindow *window)
 {
     GtkWidget *item;
-    gint list_monitor;
-    gint window_monitor;
+    GdkMonitor *list_monitor;
+    GdkMonitor *window_monitor;
     guint num;
 
     num = g_slist_length (task_lists);
@@ -211,6 +210,82 @@ static void on_task_list_orient_changed(PanelApplet *applet,
     gtk_widget_queue_resize(GTK_WIDGET(box));
 }
 
+static void
+remove_task_item (GtkWidget *item,
+                  gpointer   list)
+{
+  g_return_if_fail (TASK_IS_LIST (list));
+
+  gtk_container_remove (GTK_CONTAINER (list), item);
+}
+
+static gboolean
+on_monitors_changed (gpointer user_data)
+{
+  TaskList *list;
+  GdkWindow *window;
+  GdkMonitor *list_monitor;
+
+  list = user_data;
+  window = gtk_widget_get_window (GTK_WIDGET (list));
+
+  list_monitor = gdk_display_get_monitor_at_window (gdk_display_get_default (),
+                                                    window);
+
+  if (task_list_get_monitor (list) == list_monitor)
+    gtk_container_foreach (GTK_CONTAINER (list), remove_task_item, list);
+
+  GList *windows = wnck_screen_get_windows (list->priv->screen);
+
+  while (windows != NULL)
+    {
+      on_window_opened (list->priv->screen, windows->data, list);
+      windows = windows->next;
+    }
+
+  list->priv->size_update_event_source = 0;
+
+  return G_SOURCE_REMOVE;
+}
+
+
+static GdkFilterReturn
+window_filter_function (GdkXEvent *gdk_xevent,
+                        GdkEvent  *event,
+                        gpointer   user_data)
+{
+  TaskList *list = user_data;
+
+  XEvent *xevent = (XEvent *) gdk_xevent;
+
+  switch (xevent->type)
+    {
+      case PropertyNotify:
+        {
+          XPropertyEvent *propertyEvent;
+
+          propertyEvent = (XPropertyEvent *) xevent;
+
+          const Atom WORKAREA_ATOM = XInternAtom (propertyEvent->display,
+                                                  "_NET_WORKAREA", True);
+
+          if (propertyEvent->atom != WORKAREA_ATOM)
+            return GDK_FILTER_CONTINUE;
+
+          if (list->priv->size_update_event_source != 0)
+            return GDK_FILTER_CONTINUE;
+
+          list->priv->size_update_event_source = g_idle_add (on_monitors_changed,
+                                                             user_data);
+
+          break;
+        }
+      default:break;
+    }
+
+  return GDK_FILTER_CONTINUE;
+}
+
 static void
 task_list_dispose (GObject *object)
 {
@@ -229,6 +304,10 @@ task_list_finalize (GObject *object)
 
     task_lists = g_slist_remove (task_lists, taskList);
 
+    gdk_window_remove_filter (gtk_widget_get_window (GTK_WIDGET (taskList)),
+                              window_filter_function,
+                              taskList);
+
     G_OBJECT_CLASS (task_list_parent_class)->finalize (object);
 }
 
@@ -276,6 +355,10 @@ GtkWidget *task_list_new (WpApplet *windowPickerApplet) {
     g_signal_connect (taskList->priv->screen, "window-opened",
             G_CALLBACK (on_window_opened), taskList);
 
+    gdk_window_add_filter (gtk_widget_get_window (GTK_WIDGET (taskList)),
+                           window_filter_function,
+                           taskList);
+
     GList *windows = wnck_screen_get_windows (taskList->priv->screen);
     while (windows != NULL) {
         on_window_opened (taskList->priv->screen, windows->data, taskList);
@@ -284,15 +367,15 @@ GtkWidget *task_list_new (WpApplet *windowPickerApplet) {
     return (GtkWidget *) taskList;
 }
 
-gint
+GdkMonitor *
 task_list_get_monitor (TaskList *list)
 {
-    GdkScreen *gdk_screen;
+    GdkDisplay *gdk_display;
     GdkWindow *gdk_window;
 
+    gdk_display = gdk_display_get_default ();
     gdk_window = gtk_widget_get_window (GTK_WIDGET (list));
-    gdk_screen = gtk_widget_get_screen (GTK_WIDGET (list));
 
-    return gdk_screen_get_monitor_at_window (gdk_screen,
-                                             gdk_window);
+    return gdk_display_get_monitor_at_window (gdk_display,
+                                              gdk_window);
 }
